# -*- coding: utf-8 -*-
import unittest

from khayyam import JalaliDatetime, teh_tz, Timezone
from datetime import datetime, timedelta, time, tzinfo
from khayyam.timezones import TehranTimezone
from khayyam.jalali_date import JalaliDate
__author__ = 'vahid'


class TestJalaliDateTime(unittest.TestCase):
    
    def setUp(self):
        self.leap_year = 1375
        self.naive_jdt = JalaliDatetime(self.leap_year, 12, 30, 10, 2, 1, 3)
        self.aware_jdt = JalaliDatetime(self.leap_year, 12, 30, 10, 2, 1, 3, TehranTimezone())

    def test_instantiate(self):

        jalali_time = JalaliDatetime(1376, 10, 9, 8, 7, 6, 5)
        self.assertFalse(jalali_time is None)

        self.assertEqual(JalaliDatetime(jalali_time.todatetime()), jalali_time)
        self.assertEqual(JalaliDatetime(jalali_time), jalali_time)
        self.assertEqual(JalaliDatetime(jalali_time.date()).date(), jalali_time.date())
        self.assertEqual(JalaliDatetime(julian_day=2450674), JalaliDatetime(1376, 5, 23))

    def test_to_from_datetime(self):
        # Naive
        jalali_time1 = JalaliDatetime(self.naive_jdt.todatetime())
        self.assertEqual(self.naive_jdt, jalali_time1)
        
        # Aware
        jalali_time2 = JalaliDatetime(self.aware_jdt.todatetime())
        self.assertEqual(self.aware_jdt, jalali_time2)
        
    def test_today(self):
        dt = datetime.now().date()
        jdt = JalaliDatetime.today().date()
        self.assertEqual(jdt, JalaliDate(dt))
        
    def test_now(self):
        self.assertIsNotNone(JalaliDatetime.now())
        self.assertIsNone(JalaliDatetime.now().tzinfo)
        self.assertIsNotNone(JalaliDatetime.now(TehranTimezone()).tzinfo)
        
    def test_utcnow(self):
        jalali_utcnow = JalaliDatetime.utcnow()
        datetime_utcnow = jalali_utcnow.todatetime()
        self.assertEqual(jalali_utcnow.time(), datetime_utcnow.time())
        self.assertEqual(jalali_utcnow.timetz(), datetime_utcnow.timetz())
    
    def test_strftime_strptime(self):
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assertEqual(d1.strftime('%Q'), 'Panjshanbeh 23 Esfand 1375 12:03:45 PM')
        self.assertEqual(
            d1.strptime('Panjshanbeh 23 Esfand 1375 12:03:45 PM', '%Q'),
            d1 - timedelta(microseconds=34567))

        self.assertEqual(d1.isoformat(), '%s-12-23T12:03:45.034567' % self.leap_year)
        tz_datetime = d1.astimezone(teh_tz)
        self.assertEqual(tz_datetime.strftime('%Z'), 'Iran/Tehran')

    def test_iso_format(self):
        jalali_time = JalaliDatetime(self.leap_year, 12, 23)
        self.assertEqual(jalali_time.isoformat(), '%s-12-23T00:00:00.000000' % self.leap_year)
        jalali_time = JalaliDatetime(self.leap_year, 12, 23, tzinfo=teh_tz)
        self.assertEqual(jalali_time.isoformat(), '%s-12-23T00:00:00.000000+03:30' % self.leap_year)

    def test_algorithm(self):
        min_date = datetime(1900, 1, 1, 1, 1, 1)
        max_days = 5000  # 3000 years !
        days = 0
        while True:
            dt = min_date + timedelta(days=days)
            jd = JalaliDatetime(dt)
            # print('Processing day: %s' % jd.year)
            dt2 = jd.todatetime()
            self.assertEqual(dt, dt2)
            days += 1
            if days > max_days:
                break

    def test_add(self):
        jalali_time1 = JalaliDatetime(self.leap_year, 12, 23)
        jalali_tme2 = jalali_time1 + timedelta(10)
        self.assertEqual(jalali_tme2, JalaliDatetime(self.leap_year + 1, 1, 3))
        
    def test_sub(self):
        jalali_time1 = JalaliDatetime(self.leap_year, 12, 23)
        jalali_time2 = jalali_time1 - timedelta(10)
        self.assertEqual(jalali_time2, JalaliDatetime(self.leap_year, 12, 13))
        difference = jalali_time1 - JalaliDatetime(self.leap_year - 1, 12, 1)
        self.assertEqual(difference, timedelta(387))
        jalali_time1 = JalaliDatetime(self.leap_year, 12, 23, 4, 2, 10, 7)
        self.assertEqual(jalali_time1 - jalali_time1.date(), timedelta(hours=4, minutes=2, seconds=10, microseconds=7))

    def test_lt_gt_le_ge_ne_eg(self):
        jalali_time1 = JalaliDatetime(self.leap_year, 12, 23)
        jalali_time2 = JalaliDatetime(self.leap_year, 12, 24)
        jalali_time3 = JalaliDatetime(self.leap_year, 12, 24)
        
        self.assertTrue(jalali_time1 <= jalali_time2)
        self.assertTrue(jalali_time1 != jalali_time2)
        self.assertFalse(jalali_time1 > jalali_time2)
        self.assertTrue(jalali_time2 == jalali_time3)

    def test_replace(self):
        d1 = JalaliDatetime(1391, 12, 30)
        self.assertEqual(d1.replace(year=1395), JalaliDatetime(1395, 12, 30))
        self.assertEqual(d1.replace(month=1),   JalaliDatetime(1391, 1, 30))
        self.assertEqual(d1.replace(day=1),     JalaliDatetime(1391, 12, 1))
        self.assertRaises(ValueError, d1.replace, year=1392)

    def test_repr(self):
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assertEqual(repr(d1), 'khayyam.JalaliDatetime(1375, 12, 23, 12, 3, 45, 34567, Panjshanbeh)')

    def test_properties(self):
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assertEqual(d1.year, self.leap_year)
        self.assertEqual(d1.month, 12)
        self.assertEqual(d1.day, 23)
        self.assertEqual(d1.hour, 12)
        self.assertEqual(d1.minute, 3)
        self.assertEqual(d1.second, 45)
        self.assertEqual(d1.microsecond, 34567)
        self.assertEqual(d1.ampm(), u'ب.ظ')

        d1.year = 1361
        d1.month = 6
        d1.day = 15
        self.assertRaises(AttributeError, setattr, d1, 'hour', 16)
        self.assertRaises(AttributeError, setattr, d1, 'minute', 16)
        self.assertRaises(AttributeError, setattr, d1, 'second', 16)
        self.assertRaises(AttributeError, setattr, d1, 'microsecond', 16)
        self.assertRaises(AttributeError, setattr, d1, 'tzinfo', teh_tz)

    def test_fromtimestamp(self):
        ts = 1471628912.749938

        self.assertEqual(
            JalaliDatetime.fromtimestamp(ts),
            JalaliDatetime.combine(JalaliDatetime(1395, 5, 29), datetime.fromtimestamp(ts).time())
        )

        self.assertEqual(
            JalaliDatetime.utcfromtimestamp(ts),
            JalaliDatetime.combine(JalaliDatetime(1395, 5, 29), datetime.utcfromtimestamp(ts).time())
        )

    def test_ordinal(self):
        self.assertEqual(JalaliDatetime.min.toordinal(), 1)
        self.assertEqual(JalaliDatetime.max.toordinal(), 1160739)

        min_ = JalaliDatetime.fromordinal(JalaliDatetime.min.toordinal())
        max_ = JalaliDatetime.fromordinal(JalaliDatetime.max.toordinal())
        self.assertEqual(min_.year, 1)
        self.assertEqual(min_.month, 1)
        self.assertEqual(min_.day, 1)
        self.assertEqual(min_, JalaliDatetime.min)
        self.assertEqual(max_, JalaliDatetime.max.replace(hour=0, minute=0, second=0, microsecond=0))

    def test_combine(self):
        dt = JalaliDate(1361, 11, 6)
        t = time(10, 11, 12)
        self.assertEqual(JalaliDatetime.combine(dt, t), JalaliDatetime(1361, 11, 6, 10, 11, 12))

    def test_astimezone(self):
        d1 = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938, tzinfo=teh_tz)
        tz_fake = Timezone(timedelta(hours=2))
        self.assertEqual(d1.astimezone(teh_tz), d1)
        self.assertEqual(d1.astimezone(tz_fake), JalaliDatetime(1395, 5, 29, 19, 48, 32, 749938, tzinfo=tz_fake))

    def test_utcoffset(self):
        d1 = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938, tzinfo=teh_tz)
        naive = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938)
        self.assertEqual(d1.utcoffset(), timedelta(minutes=270))
        self.assertIsNone(naive.utcoffset())

    def test_tzname(self):
        d1 = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938, tzinfo=teh_tz)
        naive = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938)
        self.assertEqual(d1.tzname(), u'Iran/Tehran')
        self.assertIsNone(naive.tzname())

    def test_formats(self):
        d1 = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938, tzinfo=teh_tz)
        naive = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938)
        self.assertEqual(d1.localshortformat(), u'ج 29 مر 95 22:18')
        self.assertEqual(d1.localshortformatascii(), u'J 29 Mo 95 22:18')
        self.assertEqual(d1.localdatetimeformat(), u'جمعه 29 مرداد 1395 10:18:32 ب.ظ')
        self.assertEqual(d1.localdatetimeformatascii(), u'Jomeh 29 Mordad 1395 10:18:32 PM')
        self.assertEqual(d1.localtimeformat(), u'10:18:32 \u0628.\u0638')
        self.assertEqual(d1.utcoffsetformat(), u'304')
        self.assertEqual(naive.utcoffsetformat(), u'')
        self.assertEqual(d1.tznameformat(), u'Iran/Tehran')
        self.assertEqual(naive.tznameformat(), u'')

    def test_day_of_year(self):
        d1 = JalaliDatetime(1395, 5, 29, 22, 18, 32, 749938, tzinfo=teh_tz)
        self.assertEqual(d1.dayofyear(), 153)

    def test_str(self):
        d1 = JalaliDatetime(1361, 6, 15)
        self.assertEqual(
            d1.__str__(),
            d1.isoformat(sep=' ')
        )

    def test_operators(self):
        invalid_object = dict(a=2)
        d1 = JalaliDatetime(1361, 6, 15, 10, 1)
        d2 = JalaliDatetime(1361, 6, 15, 10, 2)
        self.assertRaises(TypeError, d1.__add__, invalid_object)
        self.assertRaises(TypeError, d1.__sub__, invalid_object)
        self.assertRaises(TypeError, d1.__lt__, invalid_object)
        self.assertRaises(TypeError, d1.__gt__, invalid_object)
        self.assertRaises(TypeError, d1.__eq__, invalid_object)
        self.assertFalse(d1 == 0)
        self.assertTrue(d1 == d1.todatetime())
        self.assertTrue(d1 < d2)
        self.assertTrue(d1 <= d1.copy())
        self.assertFalse(d1 > d2)
        self.assertTrue(d1 >= d1.copy())

if __name__ == '__main__':  # pragma: no cover
    unittest.main()



# -*- coding: utf-8 -*-
from khayyam import JalaliDatetime, Timezone, teh_tz
from datetime import timedelta
import unittest
__author__ = 'vahid'


class JalaliDatetimeFormatterTestCase(unittest.TestCase):

    def setUp(self):
        self.leap_year = 1375

    def assert_parse_and_format(self, jdate, fmt):
        jalali_date_str = jdate.strftime(fmt)
        parsed_dt = JalaliDatetime.strptime(jalali_date_str, fmt)
        self.assertEqual(jdate, parsed_dt)

    def test_hour(self):
        """
        Testing:
            %H            Hour (24-hour clock) as a zero padded decimal number [00, 23].
            %k            Hour (24-hour clock) as a decimal number in persian form [۱, ۲۳].
            %h            Hour (24-hour clock) as a zero padded decimal number in persian form [۰۰, ۲۳].
            %I            Hour (12-hour clock) as a zero padded decimal number [01, 12].
            %i            Hour (12-hour clock) as a zero padded decimal number in persian form [۰۱, ۱۲].
            %l            Hour (12-hour clock) as a decimal number in persian form [۱, ۱۲].
        """
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)

        # Test HOUR 24
        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%H'), u'00')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%H'), u'01')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%H'), u'12')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%H'), u'13')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%H'), u'23')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%H'), u'00')
        self.assertEqual(JalaliDatetime.strptime('08', '%H'), JalaliDatetime(hour=8))
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'2', '%H')

        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%k'), u'۰')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%k'), u'۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%k'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%k'), u'۱۳')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%k'), u'۲۳')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%k'), u'۰')
        self.assertEqual(JalaliDatetime.strptime(u'۸', '%k'), JalaliDatetime(hour=8))
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'۰۲', '%k')

        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%h'), u'۰۰')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%h'), u'۰۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%h'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%h'), u'۱۳')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%h'), u'۲۳')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%h'), u'۰۰')
        self.assertEqual(JalaliDatetime.strptime(u'۰۸', '%h'), JalaliDatetime(hour=8))
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'۲', '%h')

        # Test HOUR 12
        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%I'), u'12')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%I'), u'01')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%I'), u'12')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%I'), u'01')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%I'), u'11')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%I'), u'12')
        self.assertEqual(d1.strftime('%I'), u'12')
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'2', '%I')

        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%l'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%l'), u'۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%l'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%l'), u'۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%l'), u'۱۱')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%l'), u'۱۲')
        self.assertEqual(d1.strftime('%l'), u'۱۲')
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'۰۲', '%l')

        self.assertEqual(JalaliDatetime(1386, 12, 23).strftime('%i'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 1).strftime('%i'), u'۰۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 12).strftime('%i'), u'۱۲')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 13).strftime('%i'), u'۰۱')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 23).strftime('%i'), u'۱۱')
        self.assertEqual(
            (JalaliDatetime(1386, 12, 23, 23, 59, 59) + timedelta(seconds=1)).strftime('%i'), u'۱۲')
        self.assertEqual(d1.strftime('%i'), u'۱۲')
        self.assertRaises(ValueError, JalaliDatetime.strptime, u'۲', '%i')

        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(1000):
            d_test = d2 + timedelta(hours=i)
            self.assert_parse_and_format(d_test, '%Y-%m-%d %H:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %k:%r:%s')
            self.assert_parse_and_format(d_test, '%N-%R-%D %h:%v:%L')
            self.assert_parse_and_format(d_test, '%Y-%m-%d %p %I:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %t %i:%r:%s')
            self.assert_parse_and_format(d_test, '%N-%R-%D %p %l:%v:%L')

    def test_minute(self):
        """
        Testing:
            %M            Minute as a decimal number [00, 59].
            %r            Minute as a zero padded decimal number in persian form [۰۰, ۵۹].
            %v            Minute as a decimal number in persian form [۰, ۵۹].
        """
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6).strftime('%M'), u'06')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6).strftime('%r'), u'۰۶')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6).strftime('%v'), u'۶')
        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(1000):
            d_test = d2 + timedelta(minutes=i)
            self.assert_parse_and_format(d_test, '%Y-%m-%d %H:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %H:%r:%s')
            self.assert_parse_and_format(d_test, '%N-%R-%D %H:%v:%L')

    def test_second(self):
        """
        Testing:
            %S            Second as a decimal number [00, 59].
            %s            Second as a zero padded decimal number in persian form [۰۰, ۵۹].
            %L            Second as a decimal number in persian form [۰, ۵۹].
        """
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6, 5).strftime('%S'), u'05')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6, 5).strftime('%s'), u'۰۵')
        self.assertEqual(JalaliDatetime(1386, 12, 23, 4, 6, 5).strftime('%L'), u'۵')

        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(1000):
            d_test = d2 + timedelta(seconds=i)
            self.assert_parse_and_format(d_test, '%Y-%m-%d %H:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %H:%r:%s')
            self.assert_parse_and_format(d_test, '%N-%R-%D %H:%r:%L')

    def test_timezone(self):
        """
        Testing:
            %z            UTC offset in the form +HHMM or -HHMM (empty string if the the object is naive).
            %o            UTC offset in the form +HHMM or -HHMM (empty string if the the object is naive) in persian format i.e +۰۳:۳۰.
            %Z            Time zone name (empty string if the object is naive).
        """
        # Test Timezone
        tz_dt = JalaliDatetime(tzinfo=Timezone(timedelta(minutes=10)))
        naive_dt = tz_dt.replace(tzinfo=None)
        self.assertEqual(JalaliDatetime.strptime('00:10', '%z'), tz_dt)
        self.assertEqual(JalaliDatetime.strptime('', '%z'), naive_dt)
        self.assertEqual(tz_dt.strftime('%z'), '+00:10')
        tz_dt = JalaliDatetime(tzinfo=Timezone(timedelta(minutes=-30)))
        self.assertEqual(tz_dt.strftime('%z'), '-00:30')
        self.assertEqual(tz_dt.strftime('%o'), u'-۰۰:۳۰')
        self.assertEqual(JalaliDatetime.strptime('', '%o'), JalaliDatetime())
        self.assertEqual(JalaliDatetime.strptime('00:00', '%z'), JalaliDatetime())
        self.assertEqual(JalaliDatetime.strptime('00:01', '%z'), JalaliDatetime(tzinfo=Timezone(timedelta(minutes=1))))
        self.assertNotEqual(JalaliDatetime.strptime('04:30', '%z'), JalaliDatetime.strptime('04:31', '%z'))
        self.assertEqual(JalaliDatetime.strptime('04:30', '%z'), JalaliDatetime.strptime('04:30', '%z'))
        self.assertNotEqual(JalaliDatetime.strptime('04:30', '%z'), JalaliDatetime(tzinfo=teh_tz))
        self.assertEqual(JalaliDatetime.strptime('+04:30', '%z').utcoffset(), timedelta(hours=4.50))
        self.assertEqual(tz_dt.strftime('%z'), tz_dt.strftime('%Z'))
        self.assertEqual(naive_dt.strftime('%z'), '')
        self.assertEqual(naive_dt.strftime('%Z'), '')

        self.assertEqual(JalaliDatetime.strptime(u'۰۰:۰۰', '%o'), JalaliDatetime())
        self.assertEqual(JalaliDatetime.strptime(u'۰۰:۰۱', '%o'), JalaliDatetime(tzinfo=Timezone(timedelta(minutes=1))))
        self.assertNotEqual(JalaliDatetime.strptime(u'۰۴:۳۰', '%o'), JalaliDatetime.strptime('04:31', '%z'))
        self.assertEqual(JalaliDatetime.strptime(u'۰۴:۳۰', '%o'), JalaliDatetime.strptime('04:30', '%z'))
        self.assertNotEqual(JalaliDatetime.strptime(u'۰۴:۳۰', '%o'), JalaliDatetime(tzinfo=teh_tz))
        self.assertEqual(JalaliDatetime.strptime(u'+۰۴:۳۰', '%o').utcoffset(), timedelta(hours=4.50))
        self.assertEqual(naive_dt.strftime('%o'), '')

        self.assertEqual(
            JalaliDatetime(1394, 4, 28, 18, 14, 35, 962659, Timezone(timedelta(.3))).strftime('%Y-%m-%d %H:%M:%S.%f %z'),
            '1394-04-28 18:14:35.962659 +07:12')

        self.assertEqual(
            JalaliDatetime.strptime('1394-04-28 18:14:35.962659 +07:12', '%Y-%m-%d %H:%M:%S.%f %z'),
            JalaliDatetime(1394, 4, 28, 18, 14, 35, 962659, Timezone(timedelta(.3)))
            )

        self.assertEqual(
            JalaliDatetime.strptime(u'1394-تیر-29 دوشنبه 00:05:14.113389 +04:30', '%Y-%B-%d %A %H:%M:%S.%f %z'),
            JalaliDatetime(1394, 4, 29, 0, 5, 14, 113389, Timezone(timedelta(hours=4, minutes=30)))
        )

    def test_microseconds(self):
        """
        Testing:
            %f            Microsecond as a decimal number [0, 999999], zero-padded on the left
            %F            Microsecond as a decimal number in persian from[۰۰۰۰۰۰, ۹۹۹۹۹۹], zero-padded on the left
        """
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assert_parse_and_format(d1, '%Y-%m-%d %H:%M:%S.%f')
        self.assert_parse_and_format(d1, '%Y-%m-%d %H:%M:%S.%F')
        self.assert_parse_and_format(JalaliDatetime(1375, 12, 23, 12, 0, 0, 0), '%Y-%m-%d %p %I:%M:%S.%f')
        self.assert_parse_and_format(JalaliDatetime(1375, 12, 23, 12, 0, 0, 0), '%Y-%m-%d %p %I:%M:%S.%F')

        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(0, 100000000, 100000):
            d_test = d2 + timedelta(microseconds=i)
            self.assert_parse_and_format(d_test, '%Y-%m-%d %H:%M:%S.%f')
            self.assert_parse_and_format(d_test, '%Y-%m-%d %H:%M:%S.%F')

    def test_am_pm(self):
        """
        Testing:
            %p            Locale’s equivalent of either AM or PM in persian format [ق.ظ, ب.ظ].
            %t            Locale’s equivalent of either AM or PM in ASCII format.
        """
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assert_parse_and_format(d1, '%Y-%m-%d %p %I:%M:%S.%f')
        self.assert_parse_and_format(d1, '%Y-%m-%d %t %I:%M:%S.%F')
        self.assert_parse_and_format(JalaliDatetime(1375, 12, 23, 12, 0, 0, 0), '%Y-%m-%d %p %I:%M:%S.%F')
        self.assert_parse_and_format(JalaliDatetime(1375, 12, 23, 12, 0, 0, 0), '%Y-%m-%d %t %I:%M:%S.%F')

        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(500):
            d_test = d2 + timedelta(hours=i)
            self.assert_parse_and_format(d_test, '%Y-%m-%d %p %I:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %p %i:%r:%s')
            self.assert_parse_and_format(d_test, '%Y-%m-%d %t %I:%M:%S')
            self.assert_parse_and_format(d_test, '%N-%R-%D %t %i:%r:%s')

    def test_locale_date_time(self):
        """
        Testing:
            %c            Locale’s appropriate short date and time representation.
            %C            Locale’s appropriate date and time representation.
            %q            ASCII Locale’s appropriate short date and time representation.
            %Q            ASCII Locale’s appropriate date and time representation.
            %X            Locale’s appropriate time representation.
        """

        self.assertEqual(JalaliDatetime(1361, 6, 15).strftime('%c'), u'د ۱۵ شه ۶۱ ۰:۰')
        self.assertEqual(JalaliDatetime(1361, 6, 15, 19, 34, 2).strftime('%C'), u'دوشنبه ۱۵ شهریور ۱۳۶۱ ۰۷:۳۴:۰۲ ب.ظ')
        self.assertEqual(JalaliDatetime(1361, 6, 15, 19, 34, 2).strftime('%q'), u'D 15 Sh 61 19:34')
        self.assertEqual(JalaliDatetime(1361, 6, 15, 19, 34, 2).strftime('%Q'), u'Doshanbeh 15 Shahrivar 1361 07:34:02 PM')
        self.assertEqual(JalaliDatetime(1361, 6, 15, 19, 34, 2).strftime('%X'), u'۰۷:۳۴:۰۲ ب.ظ')
        d2 = JalaliDatetime(self.leap_year, 12, 23)
        for i in range(1000):
            d_test = d2 + timedelta(minutes=i)
            self.assert_parse_and_format(d_test, '%c')
            self.assert_parse_and_format(d_test, '%C')
            self.assert_parse_and_format(d_test, '%q')
            self.assert_parse_and_format(d_test, '%Q')
            self.assert_parse_and_format(d_test, '%Y-%m-%d %X')


if __name__ == '__main__':  # pragma: no cover
    unittest.main()




# -*- coding: utf-8 -*-
from math import floor, ceil

__author__ = 'vahid'


def get_julian_day_from_gregorian_date(year, month, day):
    is_leap = False
    if year / 4.0 == round(year / 4.0):
        if year / 100.0 == round(year / 100.0):
            if year / 400.0 == round(year / 400.0):
                is_leap = True
        else:
            is_leap = True

    if month == 2:
        max_days = 29 if is_leap else 28
        if day > max_days:
            raise ValueError('Invalid day: %s, it must be <= %s' % (day, max_days))

    year = float(year)
    month = float(month)
    day = float(day)

    # Determine JD
    if month <= 2:
        year -= 1
        month += 12

    century = floor(year / 100)
    return floor(365.25 * (year + 4716)) + \
        floor(30.6001 * (month + 1)) + \
        day + \
        (2 - century + floor(century / 4)) - \
        1524.5


def is_jalali_leap_year(year):
    return ((((((year - [473, 474][year > 0]) % 2820) + 474) + 38) * 682) % 2816) < 682


def get_days_in_jalali_year(year):
    return 366 if is_jalali_leap_year(year) else 365


def get_days_in_jalali_month(year, month):
    if 1 <= month <= 6:
        return 31
    elif 7 <= month < 12:
        return 30

    assert month == 12, 'Month must be between 1 and 12'

    # Esfand(اسفند)
    if is_jalali_leap_year(year):
        return 30
    else:
        return 29


def get_julian_day_from_jalali_date(year, month, day):
    base = year - ([473, 474][year >= 0])
    julian_year = 474 + (base % 2820)
    return day + ([((month - 1) * 30) + 6, (month - 1) * 31][month <= 7]) + floor(
        ((julian_year * 682) - 110) / 2816) + (julian_year - 1) * 365 + floor(base / 2820) * 1029983 + (1948320.5 - 1)


def get_jalali_date_from_julian_day(julian_day):
    julian_day = floor(julian_day) + 0.5
    offset = julian_day - 2121445.5  # julian_day_from_jalali(475, 1, 1) replaced by its static value
    cycle = floor(offset / 1029983)
    remaining = offset % 1029983
    if remaining == 1029982:
        year_cycle = 2820
    else:
        a1 = floor(remaining / 366)
        a2 = remaining % 366
        year_cycle = floor(((2134 * a1) + (2816 * a2) + 2815) / 1028522) + a1 + 1
    y = year_cycle + (2820 * cycle) + 474
    if y <= 0:
        y -= 1
    days_in_years = (julian_day - get_julian_day_from_jalali_date(y, 1, 1)) + 1
    m = ceil([(days_in_years - 6) / 30, days_in_years / 31][days_in_years <= 186])
    day = (julian_day - get_julian_day_from_jalali_date(y, m, 1)) + 1
    return y, m, day


def get_gregorian_date_from_julian_day(jd):
    y = 0
    m = 0

    if jd <= 0.0:
        raise ValueError('Invalid Date')

    jdm = jd + 0.5
    z = floor(jdm)
    f = jdm - z

    alpha = floor((z - 1867216.25) / 36524.25)
    b = (z + 1 + alpha - floor(alpha / 4)) + 1524
    c = floor((b - 122.1) / 365.25)
    d = floor(365.25 * c)
    e = floor((b - d) / 30.6001)
    day = b - d - floor(30.6001 * e) + f

    if e < 14:
        m = e - 1
    elif e == 14 or e == 15:
        m = e - 13

    if m > 2:
        y = c - 4716
    elif m == 1 or m == 2:
        y = c - 4715

    return int(y), int(m), int(day)


def get_jalali_date_from_gregorian_date(year, month, day):
    return get_jalali_date_from_julian_day(get_julian_day_from_gregorian_date(year, month, day))



# -*- coding: utf-8 -*-
from khayyam import SATURDAY, MONDAY
from khayyam.compat import get_unicode
from khayyam.formatting import constants as consts
from .directive import Directive, CompositeDateDirective, CompositeDatetimeDirective
from .day import \
    PersianDayDirective, \
    DayOfYearDirective, \
    PersianDayOfYearDirective
from .month import PersianMonthDirective
from .ampm import AmPmDirective, AmPmASCIIDirective
from .time import \
    PersianMicrosecondDirective, \
    PersianHour12Directive, \
    PersianHour24Directive, \
    PersianSecondDirective, \
    PersianMinuteDirective
from .year import \
    ShortYearDirective, \
    PersianYearDirective, \
    PersianShortYearDirective
from .tz import UTCOffsetDirective, TimezoneNameDirective, PersianUTCOffsetDirective

__author__ = 'vahid'

DATE_FORMAT_DIRECTIVES = [

    # YEAR
    Directive('Y', 'year', consts.YEAR_REGEX, int, lambda d: '%.4d' % d.year),
    ShortYearDirective('y', 'shortyear', consts.SHORT_YEAR_REGEX, int),
    PersianYearDirective('N', 'persianyear', consts.PERSIAN_YEAR_REGEX),
    PersianYearDirective('O', 'persianyearzeropadded', consts.PERSIAN_YEAR_ZERO_PADDED_REGEX,
                         zero_padding=True, zero_padding_length=4),
    PersianShortYearDirective('n', 'persianshortyear', consts.PERSIAN_SHORT_YEAR_REGEX, ),
    PersianShortYearDirective('u', 'persianshortyearzeropadded', consts.PERSIAN_SHORT_YEAR_ZERO_PADDED_REGEX,
                              zero_padding=True),

    # MONTH
    PersianMonthDirective('R', 'persianmonth', consts.PERSIAN_MONTH_REGEX),
    PersianMonthDirective('P', 'persianmonthzeropadded', consts.PERSIAN_MONTH_ZERO_PADDED_REGEX, zero_padding=True),
    Directive('m', 'month', consts.MONTH_REGEX, int, lambda d: '%.2d' % d.month),
    Directive('b', 'monthabbr', consts.PERSIAN_MONTH_ABBRS_REGEX, get_unicode, lambda d: d.monthabbr(),
              lambda ctx, f: ctx.update(
                  {'month': [(k, v) for k, v in consts.PERSIAN_MONTH_ABBRS.items() if v == ctx['monthabbr']][0][0]})),
    Directive('B', 'monthname', consts.PERSIAN_MONTH_NAMES_REGEX, get_unicode, lambda d: d.monthname(),
              lambda ctx, f: ctx.update(
                  {'month': [(k, v) for k, v in consts.PERSIAN_MONTH_NAMES.items() if v == ctx['monthname']][0][0]})),
    Directive('g', 'monthabbr_ascii', consts.PERSIAN_MONTH_ABBRS_ASCII_REGEX, get_unicode,
              lambda d: d.monthabbr_ascii(),
              lambda ctx, f: ctx.update({'month': [(k, v) for k, v in consts.PERSIAN_MONTH_ABBRS_ASCII.items() if
                                                   v == ctx['monthabbr_ascii']][0][0]})),
    Directive('G', 'monthnameascii', consts.PERSIAN_MONTH_NAMES_ASCII_REGEX, get_unicode,
              lambda d: d.monthnameascii(),
              lambda ctx, f: ctx.update({'month': [(k, v) for k, v in consts.PERSIAN_MONTH_NAMES_ASCII.items() if
                                                   v == ctx['monthnameascii']][0][0]})),

    # DAY
    Directive('d', 'day', consts.DAY_REGEX, int, lambda d: '%.2d' % d.day),
    PersianDayDirective('D', 'persianday', consts.PERSIAN_DAY_REGEX),
    PersianDayDirective('K', 'persiandayzeropadded', consts.PERSIAN_DAY_ZERO_PADDED_REGEX, zero_padding=True),
    DayOfYearDirective('j', 'dayofyear', consts.DAY_OF_YEAR_REGEX, int),
    PersianDayOfYearDirective('J', 'persiandayofyear', consts.PERSIAN_DAY_OF_YEAR_REGEX),
    PersianDayOfYearDirective('V', 'persiandayofyearzeropadded', consts.PERSIAN_DAY_OF_YEAR_ZERO_PADDED_REGEX,
                              zero_padding=True, zero_padding_length=3),

    # WEEK
    Directive('w', 'weekday', consts.WEEKDAY_REGEX, int, lambda d: '%d' % d.weekday()),
    Directive('W', 'weekofyear', consts.WEEK_OF_YEAR_REGEX, int, lambda d: '%.2d' % d.weekofyear(SATURDAY)),
    Directive('U', 'weekofyear', consts.WEEK_OF_YEAR_REGEX, int, lambda d: '%.2d' % d.weekofyear(MONDAY)),
    Directive('a', 'weekdayabbr', consts.PERSIAN_WEEKDAY_ABBRS_REGEX, get_unicode, lambda d: d.weekdayabbr()),
    Directive('A', 'weekdayname', consts.PERSIAN_WEEKDAY_NAMES_REGEX, get_unicode, lambda d: d.weekdayname()),
    Directive('e', 'weekdayabbrascii', consts.PERSIAN_WEEKDAY_ABBRS_ASCII_REGEX, get_unicode,
              lambda d: d.weekdayabbrascii()),
    Directive('E', 'weekdaynameascii', consts.PERSIAN_WEEKDAY_NAMES_ASCII_REGEX, get_unicode,
              lambda d: d.weekdaynameascii()),
    Directive('T', 'englishweekdaynameascii', consts.ENGLISH_WEEKDAY_NAMES_ASCII_REGEX, get_unicode,
              lambda d: d.englishweekdaynameascii()),

    # COMPOSITE
    CompositeDateDirective('x', 'localdateformat', '%s %s %s %s' % (
        consts.PERSIAN_WEEKDAY_NAMES_REGEX,
        consts.PERSIAN_DAY_REGEX,
        consts.PERSIAN_MONTH_NAMES_REGEX,
        consts.PERSIAN_YEAR_REGEX
    ), "%A %D %B %N"),

    # MISCELLANEOUS
    Directive('%', 'percent', '%', None, lambda d: '%', ),
]

TIME_FORMAT_DIRECTIVES = [

    # HOUR
    Directive('H', 'hour', consts.HOUR24_REGEX, int, lambda d: '%.2d' % d.hour),
    PersianHour24Directive('k', 'persianhour24', consts.PERSIAN_HOUR24_REGEX),
    PersianHour24Directive('h', 'persianhour24zeropadded', consts.PERSIAN_HOUR24_ZERO_PADDED_REGEX, zero_padding=True),
    Directive('I', 'hour12', consts.HOUR12_REGEX, int, lambda d: '%.2d' % d.hour12()),
    PersianHour12Directive('l', 'persianhour12', consts.PERSIAN_HOUR12_REGEX),
    PersianHour12Directive('i', 'persianhour12zeropadded', consts.PERSIAN_HOUR12_ZERO_PADDED_REGEX, zero_padding=True),

    # MINUTE
    Directive('M', 'minute', consts.MINUTE_REGEX, int, lambda d: '%.2d' % d.minute),
    PersianMinuteDirective('v', 'persianminute', consts.PERSIAN_MINUTE_REGEX),
    PersianMinuteDirective('r', 'persianminutezeropadded', consts.PERSIAN_MINUTE_ZERO_PADDED_REGEX, zero_padding=True),

    # SECOND
    Directive('S', 'second', consts.SECOND_REGEX, int, lambda d: '%.2d' % d.second),
    PersianSecondDirective('L', 'persiansecond', consts.PERSIAN_SECOND_REGEX),
    PersianSecondDirective('s', 'persiansecondzeropadded', consts.PERSIAN_SECOND_ZERO_PADDED_REGEX, zero_padding=True),

    # MICROSECOND
    Directive('f', 'microsecond', consts.MICROSECOND_REGEX, int, lambda d: '%.6d' % d.microsecond),
    PersianMicrosecondDirective('F', 'persianmicrosecond', consts.PERSIAN_MICROSECOND_REGEX),

    # AMP-PM
    AmPmDirective('p', 'ampm', consts.AM_PM_REGEX),
    AmPmASCIIDirective('t', 'ampmascii', consts.AM_PM_ASCII_REGEX),

    # TIMEZONE
    UTCOffsetDirective('z', 'utcoffset', consts.UTC_OFFSET_FORMAT_REGEX, get_unicode),
    TimezoneNameDirective('Z', 'tzname', consts.TZ_NAME_FORMAT_REGEX, get_unicode),
    PersianUTCOffsetDirective('o', 'persianutcoffset', consts.PERSIAN_UTC_OFFSET_FORMAT_REGEX),

    # COMPOSITE
    CompositeDatetimeDirective('c', 'localshortdatetimeformat', '%s %s %s %s %s:%s' % (
        consts.PERSIAN_WEEKDAY_ABBRS_REGEX,
        consts.PERSIAN_DAY_REGEX,
        consts.PERSIAN_MONTH_ABBRS_REGEX,
        consts.PERSIAN_SHORT_YEAR_REGEX,
        consts.PERSIAN_HOUR24_REGEX,
        consts.PERSIAN_MINUTE_REGEX
    ), "%a %D %b %n %k:%v"),
    CompositeDatetimeDirective('C', 'localdatetimeformat', '%s %s %s %s %s:%s:%s %s' % (
        consts.PERSIAN_WEEKDAY_NAMES_REGEX,
        consts.PERSIAN_DAY_REGEX,
        consts.PERSIAN_MONTH_NAMES_REGEX,
        consts.PERSIAN_YEAR_REGEX,
        consts.PERSIAN_HOUR12_ZERO_PADDED_REGEX,
        consts.PERSIAN_MINUTE_ZERO_PADDED_REGEX,
        consts.PERSIAN_SECOND_ZERO_PADDED_REGEX,
        consts.AM_PM_REGEX
    ), "%A %D %B %N %i:%r:%s %p"),
    CompositeDatetimeDirective('q', 'localshortdatetimeformatascii', '%s %s %s %s %s:%s' % (
        consts.PERSIAN_WEEKDAY_ABBRS_ASCII_REGEX,
        consts.DAY_REGEX,
        consts.PERSIAN_MONTH_ABBRS_ASCII_REGEX,
        consts.SHORT_YEAR_REGEX,
        consts.HOUR24_REGEX,
        consts.MINUTE_REGEX
    ), "%e %d %g %y %H:%M"),
    CompositeDatetimeDirective('Q', 'localdatetimeformatascii', '%s %s %s %s %s:%s:%s %s' % (
        consts.PERSIAN_WEEKDAY_NAMES_ASCII_REGEX,
        consts.DAY_REGEX,
        consts.PERSIAN_MONTH_NAMES_ASCII_REGEX,
        consts.YEAR_REGEX,
        consts.HOUR12_REGEX,
        consts.MINUTE_REGEX,
        consts.SECOND_REGEX,
        consts.AM_PM_ASCII_REGEX
    ), "%E %d %G %Y %I:%M:%S %t"),
    CompositeDatetimeDirective('X', 'localtimeformat', '%s:%s:%s %s' % (
        consts.PERSIAN_HOUR12_ZERO_PADDED_REGEX,
        consts.PERSIAN_MINUTE_ZERO_PADDED_REGEX,
        consts.PERSIAN_SECOND_ZERO_PADDED_REGEX,
        consts.AM_PM_REGEX
    ), "%i:%r:%s %p"),
]

DATETIME_FORMAT_DIRECTIVES = DATE_FORMAT_DIRECTIVES + TIME_FORMAT_DIRECTIVES

__all__ = [
    'Directive',
    'PersianDayDirective',
    'DayOfYearDirective',
    'PersianDayOfYearDirective',
    'PersianMonthDirective',
    'AmPmDirective',
    'AmPmASCIIDirective',
    'PersianMicrosecondDirective',
    'PersianHour12Directive',
    'PersianHour24Directive',
    'PersianSecondDirective',
    'PersianMinuteDirective',
    'ShortYearDirective',
    'PersianYearDirective',
    'PersianShortYearDirective',
    'UTCOffsetDirective',
    'TimezoneNameDirective',
    'PersianUTCOffsetDirective',
    'DATE_FORMAT_DIRECTIVES',
    'TIME_FORMAT_DIRECTIVES',
    'DATETIME_FORMAT_DIRECTIVES',
]

"""
Available directives: T
"""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from datetime import tzinfo, timedelta
from khayyam.helpers import force_encoded_string_output
__author__ = 'vahid'

ZERO_DELTA = timedelta(0)
STD_OFFSET = timedelta(minutes=210)  # Minutes
DST_OFFSET = timedelta(minutes=270)  # Minutes
DST_DIFF = DST_OFFSET - STD_OFFSET


class Timezone(tzinfo):

    def __init__(self, offset, dst_offset=None, dst_checker=None, name=None):
        assert (isinstance(offset, timedelta))
        self._offset = offset
        self._dst_offset = dst_offset
        self._dst_checker = dst_checker
        self._name = name
        super(Timezone, self).__init__()

    def fromutc(self, dt):
        if dt.tzinfo is None:
            raise ValueError('The object: %s is naive.' % dt)

        if dt.tzinfo != self:
            raise ValueError('Datetime timezone mismatch: %s != %s' % (dt.tzinfo, self))

        utc_offset = dt.utcoffset()
        dst_offset = dt.dst()
        delta = utc_offset - dst_offset  # this is self's standard offset

        if delta:
            dt += delta  # convert to standard local time
            dst_offset = dt.dst()

        if dst_offset:
            return dt + dst_offset
        else:
            return dt

    def utcoffset(self, dt):
        if self._is_dst(dt):
            return self._offset + self._dst_offset
        else:
            return self._offset

    def dst(self, dt):
        if self._dst_offset and self._is_dst(dt):
            return self._dst_offset
        else:
            return ZERO_DELTA

    def __unicode__(self):
        off = self._offset
        return '%s%.2d:%.2d%s' % (
            '+' if off.total_seconds() >= 0 else '-',
            int(off.total_seconds() / 3600),
            int((off.total_seconds() % 3600) / 60),
            (' dst:%d' % (self._dst_offset.total_seconds() / 60)) if self._dst_offset else ''
        )

    __repr__ = force_encoded_string_output(__unicode__)

    def tzname(self, dt):
        if self._name:
            return self._name
        else:
            return repr(self)

    def _is_dst(self, dt):
        if self._dst_checker:
            return self._dst_checker(dt)
        return False

    def __hash__(self):
        return hash(self._offset) ^ hash(self._dst_offset)


class TehranTimezone(Timezone):
    dst_start = (1, 1)
    dst_end = (7, 1)

    def __init__(self):
        super(TehranTimezone, self).__init__(
            timedelta(minutes=210),
            timedelta(minutes=60),
            lambda dt: (self.dst_start[0] < dt.month < self.dst_end[0]) or
                       (self.dst_start[0] == dt.month and self.dst_start[1] <= dt.day) or
                       (self.dst_end[0] == dt.month and self.dst_end[1] > dt.day),
            'Iran/Tehran'
        )



# -*- coding: utf-8 -*-
from .directive import Directive
from .persian import PersianNumberDirective
__author__ = 'vahid'


class ShortYearDirective(Directive):

    def format(self, d):
        return '%.2d' % (d.year % 100)

    def post_parser(self, ctx, formatter):
        from khayyam import JalaliDate
        ctx['year'] = int(JalaliDate.today().year / 100) * 100 + ctx['shortyear']


class PersianYearDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianYearDirective, self).format(d.year)

    def post_parser(self, ctx, formatter):
        super(PersianYearDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['year'] = ctx[self.name]


class PersianShortYearDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianShortYearDirective, self).format(d.year % 100)

    def post_parser(self, ctx, formatter):
        super(PersianShortYearDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name] is not None:
            ctx['shortyear'] = ctx[self.name]



# -*- coding: utf-8 -*-
import unittest
from khayyam.timezones import TehranTimezone, Timezone
from khayyam import JalaliDatetime
from datetime import datetime, timedelta
__author__ = 'vahid'


class TestTehTz(unittest.TestCase):
    
    def setUp(self):
        self.teh_tz = TehranTimezone()
        self.dt = datetime(2000, 1, 1)
        self.dtz = datetime(2000, 1, 1, tzinfo=self.teh_tz)
        self.another_tz = Timezone(timedelta(hours=5))
        self.zero_tz = Timezone(timedelta())

    def test_dst(self):
        naive_time = JalaliDatetime(1390, 12, 29, 1, 1, 1, 1)
        dst_aware_time = JalaliDatetime(1390, 12, 29, 1, 1, 1, 1, tzinfo=self.teh_tz)
        dst_time = dst_aware_time + timedelta(5)
        self.assertIsNotNone(dst_time.dst(), "invalid dst")
        self.assertEqual(dst_aware_time.dst(), timedelta(0), "invalid dst")
        self.assertIsNone(naive_time.dst(), "invalid dst")

    def test_repr(self):
        t = TehranTimezone()
        self.assertEqual(type(repr(t)), str)
        self.assertEqual(repr(t), '+03:30 dst:60')

    def test_fromutc(self):
        self.assertRaises(ValueError, self.teh_tz.fromutc, JalaliDatetime(1390, 12, 29, 1, 1, 1, 1, tzinfo=self.another_tz))
        self.assertRaises(ValueError, self.teh_tz.fromutc, JalaliDatetime(1390, 12, 29, 1, 1, 1, 1))
        self.assertEqual(
            self.teh_tz.fromutc(JalaliDatetime(1390, 2, 1, 1, 1, 1, 1, tzinfo=self.teh_tz)),
            JalaliDatetime(1390, 2, 1, 5, 31, 1, 1, tzinfo=self.teh_tz)
        )


if __name__ == '__main__':  # pragma: no cover
    unittest.main()        



# -*- coding: utf-8 -*-
from .constants import *
from .jalali_date import JalaliDate
from .jalali_datetime import JalaliDatetime
from .timezones import TehranTimezone, Timezone

__version__ = '3.0.4'


teh_tz = TehranTimezone()
__author__ = 'vahid'

__all__ = [
    'MINYEAR',
    'MAXYEAR',
    'SATURDAY',
    'SUNDAY',
    'MONDAY',
    'THURSDAY',
    'WEDNESDAY',
    'TUESDAY',
    'FRIDAY',
    'JalaliDate',
    'JalaliDatetime',
    'TehranTimezone',
    'Timezone',
    'teh_tz'
]



# -*- coding: utf-8 -*-
import re
from khayyam.formatting import constants as consts
from .directives import DATE_FORMAT_DIRECTIVES
__author__ = 'vahid'


class BaseFormatter(object):
    pass


class JalaliDateFormatter(BaseFormatter):

    _post_parsers = [
        'persianday',
        'persiandayzeropadded',
        'persiandayofyear',
        'persiandayofyearzeropadded',
        'persianmonth',
        'persianmonthzeropadded',
        'persianyear',
        'persianyearzeropadded',
        'persianshortyear',
        'persianshortyearzeropadded',
        'localdateformat',
        'monthabbr',
        'monthabbr_ascii',
        'monthname',
        'monthnameascii',
        'shortyear',
        'dayofyear',
    ]

    def __init__(self, format_string, directive_db=None):
        if not directive_db:
            directive_db = DATE_FORMAT_DIRECTIVES
        self.format_string = format_string
        self.directives = directive_db
        self.directives_by_key = {d.key: d for d in self.directives}
        self.directives_by_name = {d.name: d for d in self.directives}
        self._parser_regex = self._create_parser_regex()

    def _create_parser_regex(self):
        regex = u'^'
        index = 0
        for m in re.finditer(consts.FORMAT_DIRECTIVE_REGEX, self.format_string):
            directive_key = m.group()[1:]
            if directive_key not in self.directives_by_key:
                continue
            directive = self.directives_by_key[directive_key]
            if index < m.start():
                regex += self.format_string[index:m.start()]
            index = m.end()
            if directive.key == u'%':
                regex += u'%'
                continue
            regex += u'(?P<%(group_name)s>%(regexp)s)' % dict(
                group_name=directive.key,
                regexp=directive.regex
            )
        regex += self.format_string[index:]
        regex += u'$'
        return regex

    @property
    def parser_regex(self):
        return self._parser_regex

    def iter_format_directives(self):
        for m in re.finditer(consts.FORMAT_DIRECTIVE_REGEX, self.format_string):
            key = m.group()[1:]
            if key in self.directives_by_key:
                yield m, self.directives_by_key[key]

    def format(self, jalali_date):
        result = ''
        index = 0
        for match, directive in self.iter_format_directives():
            if index < match.start():
                result += self.format_string[index:match.start()]
            result += directive.format(jalali_date)
            index = match.end()
        result += self.format_string[index:]
        return result

    @staticmethod
    def filter_persian_digit(s):
        for p, e in consts.PERSIAN_DIGIT_MAPPING:
            s = s.replace(p[1], p[0])
        return s

    def _parse(self, date_string):
        m = re.match(self.parser_regex, self.filter_persian_digit(date_string))
        if not m:
            raise ValueError(u"time data '%s' does not match format '%s' with generated regex: '%s'" % (
                date_string, self.format_string, self.parser_regex))
        result = {}
        for directive_key, v in m.groupdict().items():
            directive = self.directives_by_key[directive_key]
            result[directive.name] = directive.type_(v)
        return result

    @property
    def post_parsers(self):
        return self._post_parsers

    def _parse_post_processor(self, parse_result):
        for directive_name in self.post_parsers:
            if directive_name in parse_result:
                self.directives_by_name[directive_name].post_parser(parse_result, self)

    def parse(self, date_string):
        result = self._parse(date_string)
        self._parse_post_processor(result)
        return result


class JalaliDatetimeFormatter(JalaliDateFormatter):
    _post_parsers = [
        'persianday',
        'persiandayzeropadded',
        'persiandayofyear',
        'persiandayofyearzeropadded',
        'persianmonth',
        'persianmonthzeropadded',
        'persianyear',
        'persianyearzeropadded',
        'persianshortyear',
        'persianshortyearzeropadded',
        'persianmicrosecond',
        'persianhour12',
        'persianhour12zeropadded',
        'persianhour24',
        'persianhour24zeropadded',
        'persianminute',
        'persianminutezeropadded',
        'persiansecond',
        'persiansecondzeropadded',
        'persianutcoffset',
        'localdateformat',
        'localshortdatetimeformat',
        'localshortdatetimeformatascii',
        'localdatetimeformat',
        'localdatetimeformatascii',
        'localtimeformat',
        'monthabbr',
        'monthabbr_ascii',
        'monthname',
        'monthnameascii',
        'ampm',
        'ampmascii',
        'shortyear',
        'dayofyear',
        'utcoffset'
    ]

    def __init__(self, format_string, directive_db=None):
        if not directive_db:
            from .directives import DATETIME_FORMAT_DIRECTIVES
            directive_db = DATETIME_FORMAT_DIRECTIVES
        super(JalaliDatetimeFormatter, self).__init__(format_string, directive_db=directive_db)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys
import warnings
__author__ = 'vahid'


def force_encoded_string_output(func):  # pragma: no cover
    if sys.version_info.major < 3:
        def _func(*args, **kwargs):
            return func(*args, **kwargs).encode('utf-8')
        return _func
    else:
        return func


def deprecated(func):  # pragma: no cover
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emitted
    when the function is used."""
    def new_func(*args, **kwargs):
        warnings.warn("Call to deprecated function %s." % func.__name__,
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func





# -*- coding: utf-8 -*-
import unittest
from khayyam import algorithms_c as alg_c
from khayyam import algorithms_pure as alg_p
__author__ = 'vahid'


# TODO: test with negative values
class TestCAlgorithms(unittest.TestCase):

    def test_get_julian_day_from_gregorian(self):
        self.assertRaises(ValueError, alg_p.get_julian_day_from_gregorian_date, 2016, 2, 30)
        self.assertRaises(ValueError, alg_p.get_julian_day_from_gregorian_date, 2015, 2, 29)

        self.assertRaises(ValueError, alg_c.get_julian_day_from_gregorian_date, 2016, 2, 30)
        self.assertRaises(ValueError, alg_c.get_julian_day_from_gregorian_date, 2015, 2, 29)

        self.assertRaises(ValueError, alg_c.get_julian_day_from_gregorian_date, -4713, 2, 30)
        self.assertRaises(ValueError, alg_c.get_julian_day_from_gregorian_date, -4713, 2, 29)

        self.assertEqual(
            alg_c.get_julian_day_from_gregorian_date(-4713, 11, 25),
            alg_p.get_julian_day_from_gregorian_date(-4713, 11, 25)
        )

        for i in range(3000):
            self.assertEqual(
                alg_c.get_julian_day_from_gregorian_date(i, 1, 1),
                alg_p.get_julian_day_from_gregorian_date(i, 1, 1)
            )

    def test_is_leap_year(self):
        for i in range(3000):
            self.assertEqual(
                alg_c.is_jalali_leap_year(i),
                alg_p.is_jalali_leap_year(i)
            )

    def test_days_in_year(self):
        for i in range(3000):
            self.assertEqual(
                alg_c.get_days_in_jalali_year(i),
                alg_p.get_days_in_jalali_year(i)
            )

    def test_days_in_month(self):
        for i in range(3000):
            for m in range(1, 13):
                c = alg_c.get_days_in_jalali_month(i, m)
                p = alg_p.get_days_in_jalali_month(i, m)
                self.assertEqual(c, p, "year: %s, month: %s, results: {c: %s, py: %s}" % (i, m, c, p))

    def test_julian_day_from_jalali_date(self):
        for y in range(303):
            for m in range(1, 13):
                for d in range(1, alg_c.get_days_in_jalali_month(y, m)+1):
                    self.assertEqual(
                        alg_c.get_julian_day_from_jalali_date(y, m, d),
                        alg_p.get_julian_day_from_jalali_date(y, m, d),
                        "year: %s, month: %s, day: %s" % (y, m, d)
                    )

    def test_jalali_date_from_julian_day(self):
        jd = 0
        while jd < 365 * 1000:
            jd += 1
            c = alg_c.get_jalali_date_from_julian_day(jd)
            p = alg_p.get_jalali_date_from_julian_day(jd)
            self.assertEqual(c, p, "Julian day: %s\t%s <> %s" % (jd, c, p))

    def test_gregorian_date_from_julian_day(self):
        jd = 0
        self.assertRaises(ValueError, alg_c.get_gregorian_date_from_julian_day, jd)
        self.assertRaises(ValueError, alg_p.get_gregorian_date_from_julian_day, jd)
        while jd < 365 * 200:
            jd += 1
            self.assertEqual(
                alg_c.get_gregorian_date_from_julian_day(jd),
                alg_p.get_gregorian_date_from_julian_day(jd)
            )

    def test_jalali_date_from_gregorian_date(self):
        jd = 0
        while jd < 365 * 200:
            jd += 1
            cd = alg_c.get_gregorian_date_from_julian_day(jd)
            pd = alg_p.get_gregorian_date_from_julian_day(jd)

            c = alg_c.get_jalali_date_from_gregorian_date(*cd)
            p = alg_p.get_jalali_date_from_gregorian_date(*pd)

            self.assertEqual(c, p, 'jd: %s c: %s py: %s cdate: %s pydate: %s' % (jd, c, p, cd, pd))

    def test_algorithm_import(self):
        from khayyam import algorithms
        self.assertTrue(hasattr(algorithms, 'is_jalali_leap_year'))
        self.assertTrue(hasattr(algorithms, 'get_days_in_jalali_year'))
        self.assertTrue(hasattr(algorithms, 'get_days_in_jalali_month'))
        self.assertTrue(hasattr(algorithms, 'get_julian_day_from_gregorian_date'))
        self.assertTrue(hasattr(algorithms, 'get_julian_day_from_jalali_date'))
        self.assertTrue(hasattr(algorithms, 'get_jalali_date_from_julian_day'))
        self.assertTrue(hasattr(algorithms, 'get_jalali_date_from_gregorian_date'))
        self.assertTrue(hasattr(algorithms, 'get_gregorian_date_from_julian_day'))


if __name__ == '__main__':  # pragma: no cover
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = 'vahid'

#: Minimum year supported by the library.
MINYEAR = 1

#: Maximum year supported by the library.
MAXYEAR = 3178

#: Representing the Saturday weekday.
SATURDAY = 0

#: Representing the Sunday weekday.
SUNDAY = 1

#: Representing the Monday weekday.
MONDAY = 2

#: Representing the Tuesday weekday.
TUESDAY = 3


#: Representing the Wednesday weekday.
WEDNESDAY = 4


#: Representing the Thursday weekday.
THURSDAY = 5


#: Representing the Friday weekday.
FRIDAY = 6



# -*- coding: utf-8 -*-
import sys
__author__ = 'vahid'
_py3 = sys.version_info.major == 3

if _py3:  # pragma: no cover
    # noinspection PyUnresolvedReferences
    # noinspection PyCompatibility
    from builtins import range as xrange_compat
else:
    # noinspection PyUnboundLocalVariable
    xrange_compat = xrange


# noinspection PyShadowingBuiltins
xrange = xrange_compat


def get_unicode(s):
    if _py3:  # pragma: no cover
        return str(s)
    else:  # pragma: no cover
        # noinspection PyCompatibility
        return unicode(s)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from datetime import timedelta, time, datetime
from khayyam.algorithms_pure import get_gregorian_date_from_julian_day
import khayyam
from khayyam.formatting import JalaliDatetimeFormatter, AM_PM, AM_PM_ASCII
from khayyam.helpers import force_encoded_string_output
__author__ = 'vahid'


class JalaliDatetime(khayyam.JalaliDate):
    """
    Inherited from :py:class:`khayyam.JalaliDate`.

    Represent a moment in :doc:`/persiancalendar`.

    The first parameter can be an integer,
    :py:class:`datetime.date`, :py:class:`khayyam.JalaliDate`,
    :py:class:`datetime.datetime` or :py:class:`khayyam.JalaliDatetime`.

    You may create this object by passing `julian_day` parameter.


    :param year: jalali year
    :param month: 1-12
    :param day: 1-31
    :param hour: 0-23
    :param minute: 0-59
    :param second: 0-59
    :param microsecond: 0-999999
    :param tzinfo: Timezone info
    :param julian_day:

    :type year: :py:class:`int` | :py:class:`datetime.date` | :py:class:`khayyam.JalaliDate`
    :type month: int
    :type day: int
    :type hour: int
    :type minute: int
    :type second: int
    :type microsecond: int
    :type tzinfo: :py:class:`datetime.tzinfo`
    :type julian_day: int


    :return: An object representing a moment persian calendar.
    :rtype: :py:class:`khayyam.JalaliDatetime`

    """

    #: Represent the earlier moment which supported by this class.
    min = (khayyam.MINYEAR, 1, 1)

    #: Represent the last moment which supported by this class.
    max = (khayyam.MAXYEAR, 12, 29, 23, 59, 59, 999999)

    resolution = timedelta(microseconds=1)

    def __init__(self, year=1, month=1, day=1, hour=0, minute=0, second=0,
                 microsecond=0, tzinfo=None, julian_day=None):

        if isinstance(year, JalaliDatetime):
            year, month, day, hour, minute, second, microsecond = \
                year.year, year.month, year.day, year.hour, year.minute, year.second, year.microsecond

        elif isinstance(year, datetime):
            hour, minute, second, microsecond = year.hour, year.minute, year.second, year.microsecond
            if not tzinfo:
                tzinfo = year.tzinfo

        khayyam.JalaliDate.__init__(self, year=year, month=month, day=day, julian_day=julian_day)
        self._time = time(hour, minute, second, microsecond, tzinfo)

    ##############
    # Properties #
    ##############

    @property
    def hour(self):
        """
        :getter: Returns the hour
        :type: int
        """
        return self._time.hour

    @property
    def minute(self):
        """
        :getter: Returns the minute
        :type: int
        """
        return self._time.minute

    @property
    def second(self):
        """
        :getter: Returns the second
        :type: int
        """
        return self._time.second

    @property
    def microsecond(self):
        """
        :getter: Returns the microsecond
        :type: int
        """
        return self._time.microsecond

    @property
    def tzinfo(self):
        """
        :getter: Returns the timezone info
        :type: :py:class:`datetime.tzinfo`
        """
        return self._time.tzinfo

    @staticmethod
    def formatterfactory(fmt):
        return JalaliDatetimeFormatter(fmt)

    @classmethod
    def now(cls, tz=None):
        """
        If optional argument tz is None or not specified, this is like today(), but,
        if possible, supplies more precision than can be gotten from going through a
        :py:func:`time.time()` timestamp (for example,
        this may be possible on platforms supplying the C gettimeofday() function).
        
        Else tz must be an instance of a :py:class:`datetime.tzinfo` subclass,
        and the current date and time are converted to tz's time zone.
        In this case the result is equivalent to `tz.fromutc(JalaliDatetime.utcnow().replace(tzinfo=tz))`.
        See also :py:meth:`khayyam.JalaliDate.today` and :py:meth:`khayyam.JalaliDatetime.utcnow`.

        :return: the current local date and time
        :rtype: :py:class:`khayyam.JalaliDatetime`
        """
        return cls(datetime.now(tz))

    @classmethod
    def utcnow(cls):
        """
        This is like :py:meth:`khayyam.JalaliDatetime.now`, but returns the current
        UTC date and time, as a naive datetime object.

        :return: The current UTC date and time, with tzinfo None.
        :rtype: :py:class:`khayyam.JalaliDatetime`
        """
        return cls(datetime.utcnow())

    @classmethod
    def fromtimestamp(cls, timestamp, tz=None):
        """
        If optional argument tz is None or not specified, the timestamp is converted to
        the platform's local date and time, and the returned datetime object is naive.
        
        Else tz must be an instance of a class :py:class:`datetime.tzinfo` subclass,
        and the timestamp is converted to tz's time zone. In this case the result is
        equivalent to `tz.fromutc(JalaliDatetime.utcfromtimestamp(timestamp).replace(tzinfo=tz))`.
        
        This method may raise `ValueError`, if the timestamp is out of the range of values
        supported by the platform C localtime() or gmtime() functions.
        It's common for this to be restricted to years in 1970 through 2038.

        Note that on non-POSIX systems that include leap seconds in their
        notion of a timestamp, leap seconds are ignored by fromtimestamp(), and then
        it's possible to have two timestamps differing by a second that yield
        identical datetime objects. See also :py:class:`khayyam.JalaliDatetime.utcfromtimestamp`.

        :return: The local date and time corresponding to the POSIX timestamp,
                such as is returned by :py:func:`time.time()`.

        :rtype: :py:class:`khayyam.JalaliDatetime`
        """
        return cls(datetime.fromtimestamp(timestamp, tz=tz))

    @classmethod
    def utcfromtimestamp(cls, timestamp):
        """
        This may raise ValueError, if the timestamp is
        out of the range of values supported by the platform C gmtime()
        function. It's common for this to be restricted to years in 1970
        through 2038. See also :py:meth:`khayyam.JalaliDatetime.fromtimestamp`.

        :return: The UTC datetime corresponding to the POSIX timestamp,
                with tzinfo None.

        :rtype: :py:class:`khayyam.JalaliDatetime`
        """
        return cls(datetime.utcfromtimestamp(timestamp))

    @classmethod
    def fromordinal(cls, ordinal):
        """
        Return the jalali datetime corresponding to the proleptic jalali ordinal,
        where Farvardin 1 of year 1 has ordinal 1. ValueError is
        raised unless 1 <= ordinal <= JalaliDatetime.max.toordinal(). The hour, minute, second
        and microsecond of the result are all 0, and tzinfo is None.
        """
        return cls.min + timedelta(days=ordinal - 1)

    @classmethod
    def combine(cls, date, _time):
        """
        Return a new jalali datetime object whose date members are equal to the given date object's, and whose _time
        and tzinfo members are equal to the given _time object's.
        For any datetime object d, d == datetime.combine(d.date(), d.timetz()). If date is a datetime object, its _time
        and tzinfo members are ignored.
        """
        if isinstance(date, (JalaliDatetime, khayyam.JalaliDate)):
            date = date.todate()
        return cls(datetime.combine(date, _time))

    @classmethod
    def strptime(cls, date_string, fmt):
        result = cls.formatterfactory(fmt).parse(date_string)
        result = {k: v for k, v in result.items() if k in (
            'year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond', 'tzinfo')}
        return cls(**result)

    def todatetime(self):
        arr = get_gregorian_date_from_julian_day(self.tojulianday())
        return datetime(int(arr[0]), int(arr[1]), int(arr[2]), self.hour, self.minute, self.second, self.microsecond,
                        self.tzinfo)

    def date(self):
        return khayyam.JalaliDate(self.year, self.month, self.day)

    def time(self):
        return time(self.hour, self.minute, self.second, self.microsecond)

    def timetz(self):
        return time(self.hour, self.minute, self.second, self.microsecond, self.tzinfo)

    def replace(self, year=None, month=None, day=None, hour=None,
                minute=None, second=None, microsecond=None, tzinfo=None):
        """
        Without adjusting the date the and time based tzinfo
        """
        year, month, day = self._validate(
            year if year else self.year,
            month if month else self.month,
            day if day else self.day
        )

        result = JalaliDatetime(
            year,
            month,
            day,
            self.hour if hour is None else hour,
            self.minute if minute is None else minute,
            self.second if second is None else second,
            self.microsecond if microsecond is None else microsecond,
            tzinfo if tzinfo != self.tzinfo else self.tzinfo
        )
        return result

    def astimezone(self, tz):
        if self.tzinfo is tz:
            return self
        if self.tzinfo:
            utc = self - self.utcoffset()
        else:
            utc = self
        return tz.fromutc(utc.replace(tzinfo=tz))

    def utcoffset(self):
        if self.tzinfo:
            return self.tzinfo.utcoffset(self)
        else:
            return None

    def dst(self):
        """
        If tzinfo is None, returns None, else returns self.tzinfo.dst(self), and raises an exception if the latter
        doesn’t return None, or a timedelta object representing a whole number of minutes with magnitude less than one
        day.
        """
        if self.tzinfo:
            return self.tzinfo.dst(self)
        else:
            return None

    def tzname(self):
        if self.tzinfo:
            return self.tzinfo.tzname(self)
        else:
            return None

    def isoformat(self, sep='T'):
        """
        Return a string representing the date and time in ISO 8601 format, YYYY-MM-DDTHH:MM:SS.mmmmmm or, if
        microsecond is 0, YYYY-MM-DDTHH:MM:SS

        If utcoffset() does not return None, a 6-character string is appended, giving the UTC offset in (signed) hours
        and minutes: YYYY-MM-DDTHH:MM:SS.mmmmmm+HH:MM or, if microsecond is 0 YYYY-MM-DDTHH:MM:SS+HH:MM
        """
        return self.strftime('%Y-%m-%d' + sep + '%H:%M:%S.%f%z')

    def localshortformat(self):
        return self.strftime('%a %d %b %y %H:%M')

    def localshortformatascii(self):
        return self.strftime('%e %d %g %y %H:%M')

    def localdatetimeformat(self):
        return self.strftime('%A %d %B %Y %I:%M:%S %p')

    def localdatetimeformatascii(self):
        return self.strftime('%E %d %G %Y %I:%M:%S %t')

    def localtimeformat(self):
        return self.strftime('%I:%M:%S %p')

    def hour12(self):
        res = self.hour
        if res > 12:
            res -= 12
        elif res == 0:
            res = 12
        return res

    def ampm(self):
        if self.hour < 12:
            return AM_PM[0]
        return AM_PM[1]

    def ampmascii(self):
        if self.hour < 12:
            return AM_PM_ASCII[0]
        return AM_PM_ASCII[1]

    def utcoffsetformat(self):
        if self.tzinfo:
            td = self.utcoffset()
            _minutes = td.seconds / 60
            hours = _minutes / 60
            minutes = _minutes % 60
            return '%d%d' % (minutes, hours)
        return ''

    def tznameformat(self):
        return self.tzname() or ''

    def dayofyear(self):
        return (self.date() - khayyam.JalaliDate(self.year, 1, 1)).days + 1

    def __unicode__(self):
        return 'khayyam.JalaliDatetime(%s, %s, %s, %s, %s, %s, %s%s, %s)' % (
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
            ', tzinfo=%s' % self.tzinfo.__unicode__() if self.tzinfo else '',
            self.weekdaynameascii()
        )

    __repr__ = force_encoded_string_output(__unicode__)

    def __str__(self):
        return self.isoformat(sep=' ')

    def __add__(self, x):
        if isinstance(x, timedelta):
            return JalaliDatetime(x + self.todatetime())

        raise TypeError('JalaliDatetime object can added by timedelta or JalaliDate object')

    def __sub__(self, x):
        if isinstance(x, timedelta):
            return JalaliDatetime(self.todatetime() - x)
        elif isinstance(x, JalaliDatetime):
            return self.todatetime() - x.todatetime()
        elif isinstance(x, khayyam.JalaliDate):
            return self.todatetime() - JalaliDatetime(x).todatetime()

        raise TypeError('JalaliDatetime object can added by timedelta, JalaliDatetime or JalaliDate object')

    def __lt__(self, x):
        self._ensure_jalali_datetime(x)
        return self.todatetime() < x.todatetime()

    def __le__(self, x):
        self._ensure_jalali_datetime(x)
        return self.todatetime() <= x.todatetime()

    def __hash__(self):
        return super(JalaliDatetime, self).__hash__() ^ \
            hash(self.hour) ^ \
            hash(self.minute) ^ \
            hash(self.second) ^ \
            hash(self.microsecond) ^ \
            hash(self.tzinfo)

    def __eq__(self, x):
        if not x:
            return False
        if isinstance(x, datetime):
            return self.todatetime().__eq__(x)
        elif isinstance(x, JalaliDatetime):
            return hash(self) == hash(x)
        else:
            raise TypeError('Comparison only allowed with JalaliDatetime and datetime.datetime objects.')

    def __gt__(self, x):
        self._ensure_jalali_datetime(x)
        return self.todatetime() > x.todatetime()

    def __ge__(self, x):
        self._ensure_jalali_datetime(x)
        return self.todatetime() >= x.todatetime()

    @staticmethod
    def _ensure_jalali_datetime(x):
        if not isinstance(x, JalaliDatetime):
            raise TypeError('Comparison just allow with JalaliDatetime')

    def copy(self):
        """
        It's equivalent to:

            >>> source_date = JalaliDatetime(1394, 3, 24, 10, 2, 3, 999999)
            >>> JalaliDatetime(source_date.year, source_date.month, source_date.day, source_date.hour, source_date.minute, source_date.second, source_date.microsecond)
            khayyam.JalaliDatetime(1394, 3, 24, 10, 2, 3, 999999, Yekshanbeh)

        :return: A Copy of the current instance.
        :rtype: :py:class:`khayyam.JalaiDatetime`
        """
        return JalaliDatetime(
            self.year, self.month, self.day,
            self.hour, self.minute, self.second, self.microsecond,
            tzinfo=self.tzinfo
        )


# # Class attributes
JalaliDatetime.min = JalaliDatetime(*JalaliDatetime.min)
JalaliDatetime.max = JalaliDatetime(*JalaliDatetime.max)



# -*- coding: utf-8 -*-
import os
import re
import sys
import warnings
from setuptools import setup, find_packages, Extension
import traceback

__author__ = 'vahid'

# reading package version (same way sqlalchemy does)
with open(os.path.join(os.path.dirname(__file__), 'khayyam', '__init__.py')) as v_file:
    package_version = re.compile(r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)

if sys.version_info >= (3, ):
    def readme(fn):
        with open(fn, encoding='UTF-8') as f:
            return f.read()
else:
    def readme(fn):
        with open(fn) as f:
            return f.read()


setup_args = dict(
    name="Khayyam",
    version=package_version,
    author="Vahid Mardani",
    author_email="vahid.mardani@gmail.com",
    url="http://khayyam.dobisel.com",
    description="Persian Date & Time library (aka: Jalali Calendar) with timezone, DST"
                "(daylight-saving), full formatting & parsing support for python 2 & 3 including c extention",
    zip_safe=True,
    keywords="Khayyam persian jalali date time datetime conversion",
    long_description=readme(os.path.join(os.path.dirname(__file__), 'README.rst')),
    license="GPLv3",
    packages=find_packages(),
    test_suite="khayyam.tests",
    tests_require=[
        'rtl'
    ],
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        "Intended Audience :: Developers",
        "Natural Language :: Persian",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Localization"
    ],
)


def run_setup(with_extensions=True):
    if with_extensions:

        setup_args['ext_modules'] = [
            Extension(
                "khayyam.algorithms_c",
                sources=["khayyam/algorithms_c.c"]
            )
        ]

    elif 'ext_modules' in setup_args:
        del setup_args['ext_modules']

    setup(**setup_args)


def warning_c_extention():
    warnings.warn(
        '\n%s\n%s\n%s' % (
            '#' * 40,
            "WARNING: The C extension could not be compiled.\n"
            "WARNING: Speedups are not enabled.",
            '#' * 40
        ))


try:
    run_setup()

# noinspection PyBroadException
except:
    traceback.print_exc()
    warning_c_extention()
    run_setup(with_extensions=False)
    warning_c_extention()



# -*- coding: utf-8 -*-
import warnings

__author__ = 'vahid'

try:
    from .algorithms_c import \
        is_jalali_leap_year, \
        get_julian_day_from_gregorian_date, \
        get_days_in_jalali_year, \
        get_days_in_jalali_month, \
        get_julian_day_from_jalali_date, \
        get_jalali_date_from_julian_day, \
        get_gregorian_date_from_julian_day, \
        get_jalali_date_from_gregorian_date

except ImportError:  # pragma: no cover
    warnings.warn(
        "################################################################################\n"
        "The C extension is not available. Switching to fallback python pure algorithms, \n"
        "so it's about 1000X slower than C implementation of the algorithms.             \n"
        "################################################################################\n"
    )
    from .algorithms_pure import \
        is_jalali_leap_year, \
        get_julian_day_from_gregorian_date, \
        get_days_in_jalali_year, \
        get_days_in_jalali_month, \
        get_julian_day_from_jalali_date, \
        get_jalali_date_from_julian_day, \
        get_gregorian_date_from_julian_day, \
        get_jalali_date_from_gregorian_date



